@@ -7,8 +7,11 @@ namespace SourceGit.Converters
 {
     public static class ListConverters
     {
+        public static readonly FuncValueConverter<IList, string> Count =
+            new FuncValueConverter<IList, string>(v => v == null ? "0" : $"{v.Count}");
+
         public static readonly FuncValueConverter<IList, string> ToCount =
-            new FuncValueConverter<IList, string>(v => v == null ? " (0)" : $" ({v.Count})");
+            new FuncValueConverter<IList, string>(v => v == null ? "(0)" : $"({v.Count})");
 
         public static readonly FuncValueConverter<IList, bool> IsNullOrEmpty =
             new FuncValueConverter<IList, bool>(v => v == null || v.Count == 0);
