@@ -1,4 +1,4 @@
-﻿using System.Collections.Generic;
+using System.Collections.Generic;
 using System.Text;
 using System.Text.RegularExpressions;
 
@@ -147,7 +147,7 @@ namespace SourceGit.Models
         public void GenerateNewPatchFromSelection(Change change, string fileBlobGuid, TextDiffSelection selection, bool revert, string output)
         {
             var isTracked = !string.IsNullOrEmpty(fileBlobGuid);
-            var fileGuid = isTracked ? fileBlobGuid.Substring(0, 8) : "00000000";
+            var fileGuid = isTracked ? fileBlobGuid : "00000000";
 
             var builder = new StringBuilder();
             builder.Append("diff --git a/").Append(change.Path).Append(" b/").Append(change.Path).Append('\n');
