@@ -1,4 +1,4 @@
-﻿using System;
+using System;
 using System.Collections.Generic;
 using System.Text;
 
@@ -32,12 +32,24 @@ namespace SourceGit.Models
             set;
         } = false;
 
-        public bool OnlyHighlighCurrentBranchInHistories
+        public bool OnlyHighlightCurrentBranchInHistories
         {
             get;
             set;
         } = false;
 
+        public BranchSortMode LocalBranchSortMode
+        {
+            get;
+            set;
+        } = BranchSortMode.Name;
+
+        public BranchSortMode RemoteBranchSortMode
+        {
+            get;
+            set;
+        } = BranchSortMode.Name;
+
         public TagSortMode TagSortMode
         {
             get;
@@ -68,18 +80,6 @@ namespace SourceGit.Models
             set;
         } = true;
 
-        public bool FetchWithoutTagsOnPull
-        {
-            get;
-            set;
-        } = false;
-
-        public bool FetchAllBranchesOnPull
-        {
-            get;
-            set;
-        } = true;
-
         public bool CheckSubmodulesOnPush
         {
             get;
@@ -110,6 +110,12 @@ namespace SourceGit.Models
             set;
         } = true;
 
+        public bool UpdateSubmodulesOnCheckoutBranch
+        {
+            get;
+            set;
+        } = true;
+
         public AvaloniaList<Filter> HistoriesFilters
         {
             get;
@@ -182,7 +188,7 @@ namespace SourceGit.Models
             set;
         } = false;
 
-        public string PreferedOpenAIService
+        public string PreferredOpenAIService
         {
             get;
             set;
@@ -326,28 +332,28 @@ namespace SourceGit.Models
                     if (filter.Mode == FilterMode.Included)
                         includedRefs.Add(filter.Pattern);
                     else if (filter.Mode == FilterMode.Excluded)
-                        excludedBranches.Add($"--exclude=\"{filter.Pattern.Substring(11)}\" --decorate-refs-exclude=\"{filter.Pattern}\"");
+                        excludedBranches.Add($"--exclude=\"{filter.Pattern.AsSpan(11)}\" --decorate-refs-exclude=\"{filter.Pattern}\"");
                 }
                 else if (filter.Type == FilterType.LocalBranchFolder)
                 {
                     if (filter.Mode == FilterMode.Included)
-                        includedRefs.Add($"--branches={filter.Pattern.Substring(11)}/*");
+                        includedRefs.Add($"--branches={filter.Pattern.AsSpan(11)}/*");
                     else if (filter.Mode == FilterMode.Excluded)
-                        excludedBranches.Add($"--exclude=\"{filter.Pattern.Substring(11)}/*\" --decorate-refs-exclude=\"{filter.Pattern}/*\"");
+                        excludedBranches.Add($"--exclude=\"{filter.Pattern.AsSpan(11)}/*\" --decorate-refs-exclude=\"{filter.Pattern}/*\"");
                 }
                 else if (filter.Type == FilterType.RemoteBranch)
                 {
                     if (filter.Mode == FilterMode.Included)
                         includedRefs.Add(filter.Pattern);
                     else if (filter.Mode == FilterMode.Excluded)
-                        excludedRemotes.Add($"--exclude=\"{filter.Pattern.Substring(13)}\" --decorate-refs-exclude=\"{filter.Pattern}\"");
+                        excludedRemotes.Add($"--exclude=\"{filter.Pattern.AsSpan(13)}\" --decorate-refs-exclude=\"{filter.Pattern}\"");
                 }
                 else if (filter.Type == FilterType.RemoteBranchFolder)
                 {
                     if (filter.Mode == FilterMode.Included)
-                        includedRefs.Add($"--remotes={filter.Pattern.Substring(13)}/*");
+                        includedRefs.Add($"--remotes={filter.Pattern.AsSpan(13)}/*");
                     else if (filter.Mode == FilterMode.Excluded)
-                        excludedRemotes.Add($"--exclude=\"{filter.Pattern.Substring(13)}/*\" --decorate-refs-exclude=\"{filter.Pattern}/*\"");
+                        excludedRemotes.Add($"--exclude=\"{filter.Pattern.AsSpan(13)}/*\" --decorate-refs-exclude=\"{filter.Pattern}/*\"");
                 }
                 else if (filter.Type == FilterType.Tag)
                 {
@@ -399,6 +405,7 @@ namespace SourceGit.Models
 
         public void PushCommitMessage(string message)
         {
+            message = message.Trim().ReplaceLineEndings("\n");
             var existIdx = CommitMessages.IndexOf(message);
             if (existIdx == 0)
                 return;
@@ -446,5 +453,19 @@ namespace SourceGit.Models
             if (act != null)
                 CustomActions.Remove(act);
         }
+
+        public void MoveCustomActionUp(CustomAction act)
+        {
+            var idx = CustomActions.IndexOf(act);
+            if (idx > 0)
+                CustomActions.Move(idx - 1, idx);
+        }
+
+        public void MoveCustomActionDown(CustomAction act)
+        {
+            var idx = CustomActions.IndexOf(act);
+            if (idx < CustomActions.Count - 1)
+                CustomActions.Move(idx + 1, idx);
+        }
     }
 }
