@@ -11,6 +11,6 @@ namespace SourceGit.Models
         public ulong Time { get; set; } = 0;
         public string Message { get; set; } = "";
 
-        public string TimeStr => DateTime.UnixEpoch.AddSeconds(Time).ToLocalTime().ToString(DateTimeFormat.Actived.DateTime);
+        public string TimeStr => DateTime.UnixEpoch.AddSeconds(Time).ToLocalTime().ToString(DateTimeFormat.Active.DateTime);
     }
 }
