@@ -8,6 +8,7 @@ using System.Text;
 
 using Avalonia;
 using Avalonia.Controls;
+using Avalonia.Platform;
 using Avalonia.Threading;
 
 namespace SourceGit.Native
@@ -15,16 +16,12 @@ namespace SourceGit.Native
     [SupportedOSPlatform("windows")]
     internal class Windows : OS.IBackend
     {
-        [StructLayout(LayoutKind.Sequential)]
-        internal struct RTL_OSVERSIONINFOEX
+        internal struct RECT
         {
-            internal uint dwOSVersionInfoSize;
-            internal uint dwMajorVersion;
-            internal uint dwMinorVersion;
-            internal uint dwBuildNumber;
-            internal uint dwPlatformId;
-            [MarshalAs(UnmanagedType.ByValTStr, SizeConst = 128)]
-            internal string szCSDVersion;
+            public int left;
+            public int top;
+            public int right;
+            public int bottom;
         }
 
         [StructLayout(LayoutKind.Sequential)]
@@ -36,9 +33,6 @@ namespace SourceGit.Native
             public int cyBottomHeight;
         }
 
-        [DllImport("ntdll.dll")]
-        private static extern int RtlGetVersion(ref RTL_OSVERSIONINFOEX lpVersionInformation);
-
         [DllImport("dwmapi.dll")]
         private static extern int DwmExtendFrameIntoClientArea(IntPtr hwnd, ref MARGINS margins);
 
@@ -54,18 +48,79 @@ namespace SourceGit.Native
         [DllImport("shell32.dll", CharSet = CharSet.Unicode, SetLastError = false)]
         private static extern int SHOpenFolderAndSelectItems(IntPtr pidlFolder, int cild, IntPtr apidl, int dwFlags);
 
+        [DllImport("user32.dll")]
+        private static extern bool GetWindowRect(IntPtr hwnd, out RECT lpRect);
+
         public void SetupApp(AppBuilder builder)
         {
             // Fix drop shadow issue on Windows 10
-            RTL_OSVERSIONINFOEX v = new RTL_OSVERSIONINFOEX();
-            v.dwOSVersionInfoSize = (uint)Marshal.SizeOf<RTL_OSVERSIONINFOEX>();
-            if (RtlGetVersion(ref v) == 0 && (v.dwMajorVersion < 10 || v.dwBuildNumber < 22000))
+            if (!OperatingSystem.IsWindowsVersionAtLeast(10, 22000, 0))
             {
                 Window.WindowStateProperty.Changed.AddClassHandler<Window>((w, _) => FixWindowFrameOnWin10(w));
                 Control.LoadedEvent.AddClassHandler<Window>((w, _) => FixWindowFrameOnWin10(w));
             }
         }
 
+        public void SetupWindow(Window window)
+        {
+            window.ExtendClientAreaChromeHints = ExtendClientAreaChromeHints.NoChrome;
+            window.ExtendClientAreaToDecorationsHint = true;
+            window.Classes.Add("fix_maximized_padding");
+
+            Win32Properties.AddWndProcHookCallback(window, (IntPtr hWnd, uint msg, IntPtr wParam, IntPtr lParam, ref bool handled) =>
+            {
+                // Custom WM_NCHITTEST
+                if (msg == 0x0084)
+                {
+                    handled = true;
+
+                    if (window.WindowState == WindowState.FullScreen || window.WindowState == WindowState.Maximized)
+                        return 1; // HTCLIENT
+
+                    var p = IntPtrToPixelPoint(lParam);
+                    GetWindowRect(hWnd, out var rcWindow);
+
+                    var borderThickness = (int)(4 * window.RenderScaling);
+                    int y = 1;
+                    int x = 1;
+                    if (p.X >= rcWindow.left && p.X < rcWindow.left + borderThickness)
+                        x = 0;
+                    else if (p.X < rcWindow.right && p.X >= rcWindow.right - borderThickness)
+                        x = 2;
+
+                    if (p.Y >= rcWindow.top && p.Y < rcWindow.top + borderThickness)
+                        y = 0;
+                    else if (p.Y < rcWindow.bottom && p.Y >= rcWindow.bottom - borderThickness)
+                        y = 2;
+
+                    var zone = y * 3 + x;
+                    switch (zone)
+                    {
+                        case 0:
+                            return 13; // HTTOPLEFT
+                        case 1:
+                            return 12; // HTTOP
+                        case 2:
+                            return 14; // HTTOPRIGHT
+                        case 3:
+                            return 10; // HTLEFT
+                        case 4:
+                            return 1;  // HTCLIENT
+                        case 5:
+                            return 11; // HTRIGHT
+                        case 6:
+                            return 16; // HTBOTTOMLEFT
+                        case 7:
+                            return 15; // HTBOTTOM
+                        default:
+                            return 17; // HTBOTTOMRIGHT
+                    }
+                }
+
+                return IntPtr.Zero;
+            });
+        }
+
         public string FindGitExecutable()
         {
             var reg = Microsoft.Win32.RegistryKey.OpenBaseKey(
@@ -228,6 +283,12 @@ namespace SourceGit.Native
             }, DispatcherPriority.Render);
         }
 
+        private PixelPoint IntPtrToPixelPoint(IntPtr param)
+        {
+            var v = IntPtr.Size == 4 ? param.ToInt32() : (int)(param.ToInt64() & 0xFFFFFFFF);
+            return new PixelPoint((short)(v & 0xffff), (short)(v >> 16));
+        }
+
         #region EXTERNAL_EDITOR_FINDER
         private string FindVSCode()
         {
@@ -290,10 +351,10 @@ namespace SourceGit.Native
                     Microsoft.Win32.RegistryView.Registry64);
 
             // VSCodium (system)
-            var systemVScodium = localMachine.OpenSubKey(@"SOFTWARE\Microsoft\Windows\CurrentVersion\Uninstall\{88DA3577-054F-4CA1-8122-7D820494CFFB}_is1");
-            if (systemVScodium != null)
+            var systemVSCodium = localMachine.OpenSubKey(@"SOFTWARE\Microsoft\Windows\CurrentVersion\Uninstall\{88DA3577-054F-4CA1-8122-7D820494CFFB}_is1");
+            if (systemVSCodium != null)
             {
-                return systemVScodium.GetValue("DisplayIcon") as string;
+                return systemVSCodium.GetValue("DisplayIcon") as string;
             }
 
             var currentUser = Microsoft.Win32.RegistryKey.OpenBaseKey(
