@@ -1,13 +1,11 @@
 ﻿using System;
 using System.Collections.Generic;
-using System.Globalization;
 using System.IO;
 using System.Text.RegularExpressions;
 using System.Threading;
 using System.Threading.Tasks;
 
 using Avalonia.Controls;
-using Avalonia.Media.Imaging;
 using Avalonia.Platform.Storage;
 using Avalonia.Threading;
 
@@ -15,7 +13,7 @@ using CommunityToolkit.Mvvm.ComponentModel;
 
 namespace SourceGit.ViewModels
 {
-    public partial class CommitDetail : ObservableObject
+    public partial class CommitDetail : ObservableObject, IDisposable
     {
         public int ActivePageIndex
         {
@@ -56,7 +54,7 @@ namespace SourceGit.ViewModels
         {
             get;
             private set;
-        } = [];
+        }
 
         public List<string> Children
         {
@@ -103,16 +101,20 @@ namespace SourceGit.ViewModels
             set
             {
                 if (SetProperty(ref _searchChangeFilter, value))
-                {
                     RefreshVisibleChanges();
-                }
             }
         }
 
+        public string ViewRevisionFilePath
+        {
+            get => _viewRevisionFilePath;
+            private set => SetProperty(ref _viewRevisionFilePath, value);
+        }
+
         public object ViewRevisionFileContent
         {
             get => _viewRevisionFileContent;
-            set => SetProperty(ref _viewRevisionFileContent, value);
+            private set => SetProperty(ref _viewRevisionFileContent, value);
         }
 
         public string RevisionFileSearchFilter
@@ -137,7 +139,7 @@ namespace SourceGit.ViewModels
             WebLinks = Models.CommitLink.Get(repo.Remotes);
         }
 
-        public void Cleanup()
+        public void Dispose()
         {
             _repo = null;
             _commit = null;
@@ -149,6 +151,7 @@ namespace SourceGit.ViewModels
             _diffContext = null;
             _viewRevisionFileContent = null;
             _cancellationSource = null;
+            _requestingRevisionFiles = false;
             _revisionFiles = null;
             _revisionFileSearchSuggestion = null;
         }
@@ -192,10 +195,13 @@ namespace SourceGit.ViewModels
         {
             if (file == null)
             {
+                ViewRevisionFilePath = string.Empty;
                 ViewRevisionFileContent = null;
                 return;
             }
 
+            ViewRevisionFilePath = file.Path;
+
             switch (file.Type)
             {
                 case Models.ObjectType.Blob:
@@ -204,14 +210,11 @@ namespace SourceGit.ViewModels
                         var isBinary = new Commands.IsBinary(_repo.FullPath, _commit.SHA, file.Path).Result();
                         if (isBinary)
                         {
-                            var ext = Path.GetExtension(file.Path);
-                            if (IMG_EXTS.Contains(ext))
+                            var imgDecoder = ImageSource.GetDecoder(file.Path);
+                            if (imgDecoder != Models.ImageDecoder.None)
                             {
-                                var stream = Commands.QueryFileContent.Run(_repo.FullPath, _commit.SHA, file.Path);
-                                var fileSize = stream.Length;
-                                var bitmap = fileSize > 0 ? new Bitmap(stream) : null;
-                                var imageType = ext!.Substring(1).ToUpper(CultureInfo.CurrentCulture);
-                                var image = new Models.RevisionImageFile() { Image = bitmap, FileSize = fileSize, ImageType = imageType };
+                                var source = ImageSource.FromRevision(_repo.FullPath, _commit.SHA, file.Path, imgDecoder);
+                                var image = new Models.RevisionImageFile(file.Path, source.Bitmap, source.Size);
                                 Dispatcher.UIThread.Invoke(() => ViewRevisionFileContent = image);
                             }
                             else
@@ -226,13 +229,20 @@ namespace SourceGit.ViewModels
 
                         var contentStream = Commands.QueryFileContent.Run(_repo.FullPath, _commit.SHA, file.Path);
                         var content = new StreamReader(contentStream).ReadToEnd();
-                        var matchLFS = REG_LFS_FORMAT().Match(content);
-                        if (matchLFS.Success)
+                        var lfs = Models.LFSObject.Parse(content);
+                        if (lfs != null)
                         {
-                            var obj = new Models.RevisionLFSObject() { Object = new Models.LFSObject() };
-                            obj.Object.Oid = matchLFS.Groups[1].Value;
-                            obj.Object.Size = long.Parse(matchLFS.Groups[2].Value);
-                            Dispatcher.UIThread.Invoke(() => ViewRevisionFileContent = obj);
+                            var imgDecoder = ImageSource.GetDecoder(file.Path);
+                            if (imgDecoder != Models.ImageDecoder.None)
+                            {
+                                var combined = new RevisionLFSImage(_repo.FullPath, file.Path, lfs, imgDecoder);
+                                Dispatcher.UIThread.Invoke(() => ViewRevisionFileContent = combined);
+                            }
+                            else
+                            {
+                                var rlfs = new Models.RevisionLFSObject() { Object = lfs };
+                                Dispatcher.UIThread.Invoke(() => ViewRevisionFileContent = rlfs);
+                            }
                         }
                         else
                         {
@@ -244,30 +254,16 @@ namespace SourceGit.ViewModels
                 case Models.ObjectType.Commit:
                     Task.Run(() =>
                     {
-                        var submoduleRoot = Path.Combine(_repo.FullPath, file.Path);
+                        var submoduleRoot = Path.Combine(_repo.FullPath, file.Path).Replace('\\', '/').Trim('/');
                         var commit = new Commands.QuerySingleCommit(submoduleRoot, file.SHA).Result();
-                        if (commit != null)
+                        var message = commit != null ? new Commands.QueryCommitFullMessage(submoduleRoot, file.SHA).Result() : null;
+                        var module = new Models.RevisionSubmodule()
                         {
-                            var body = new Commands.QueryCommitFullMessage(submoduleRoot, file.SHA).Result();
-                            var submodule = new Models.RevisionSubmodule()
-                            {
-                                Commit = commit,
-                                FullMessage = new Models.CommitFullMessage { Message = body }
-                            };
+                            Commit = commit ?? new Models.Commit() { SHA = _commit.SHA },
+                            FullMessage = new Models.CommitFullMessage { Message = message }
+                        };
 
-                            Dispatcher.UIThread.Invoke(() => ViewRevisionFileContent = submodule);
-                        }
-                        else
-                        {
-                            Dispatcher.UIThread.Invoke(() =>
-                            {
-                                ViewRevisionFileContent = new Models.RevisionSubmodule()
-                                {
-                                    Commit = new Models.Commit() { SHA = file.SHA },
-                                    FullMessage = null,
-                                };
-                            });
-                        }
+                        Dispatcher.UIThread.Invoke(() => ViewRevisionFileContent = module);
                     });
                     break;
                 default:
@@ -291,7 +287,7 @@ namespace SourceGit.ViewModels
                 ev.Handled = true;
             };
 
-            var fullPath = Path.Combine(_repo.FullPath, change.Path);
+            var fullPath = Native.OS.GetAbsPath(_repo.FullPath, change.Path);
             var explore = new MenuItem();
             explore.Header = App.Text("RevealFile");
             explore.Icon = App.CreateMenuIcon("Icons.Explore");
@@ -335,7 +331,7 @@ namespace SourceGit.ViewModels
                 options.DefaultExtension = ".patch";
                 options.FileTypeChoices = [new FilePickerFileType("Patch File") { Patterns = ["*.patch"] }];
 
-                var baseRevision = _commit.Parents.Count == 0 ? "4b825dc642cb6eb9a060e54bf8d69288fbee4904" : _commit.Parents[0];
+                var baseRevision = _commit.Parents.Count == 0 ? Models.Commit.EmptyTreeSHA1 : _commit.Parents[0];
                 var storageFile = await storageProvider.SaveFilePickerAsync(options);
                 if (storageFile != null)
                 {
@@ -362,11 +358,9 @@ namespace SourceGit.ViewModels
                 var resetToThisRevision = new MenuItem();
                 resetToThisRevision.Header = App.Text("ChangeCM.CheckoutThisRevision");
                 resetToThisRevision.Icon = App.CreateMenuIcon("Icons.File.Checkout");
-                resetToThisRevision.Click += (_, ev) =>
+                resetToThisRevision.Click += async (_, ev) =>
                 {
-                    var log = _repo.CreateLog($"Reset File to '{_commit.SHA}'");
-                    new Commands.Checkout(_repo.FullPath).Use(log).FileWithRevision(change.Path, $"{_commit.SHA}");
-                    log.Complete();
+                    await ResetToThisRevision(change.Path);
                     ev.Handled = true;
                 };
 
@@ -374,14 +368,9 @@ namespace SourceGit.ViewModels
                 resetToFirstParent.Header = App.Text("ChangeCM.CheckoutFirstParentRevision");
                 resetToFirstParent.Icon = App.CreateMenuIcon("Icons.File.Checkout");
                 resetToFirstParent.IsEnabled = _commit.Parents.Count > 0;
-                resetToFirstParent.Click += (_, ev) =>
+                resetToFirstParent.Click += async (_, ev) =>
                 {
-                    var log = _repo.CreateLog($"Reset File to '{_commit.SHA}~1'");
-                    if (change.Index == Models.ChangeState.Renamed)
-                        new Commands.Checkout(_repo.FullPath).Use(log).FileWithRevision(change.OriginalPath, $"{_commit.SHA}~1");
-
-                    new Commands.Checkout(_repo.FullPath).Use(log).FileWithRevision(change.Path, $"{_commit.SHA}~1");
-                    log.Complete();
+                    await ResetToParentRevision(change);
                     ev.Handled = true;
                 };
 
@@ -389,8 +378,7 @@ namespace SourceGit.ViewModels
                 menu.Items.Add(resetToFirstParent);
                 menu.Items.Add(new MenuItem { Header = "-" });
 
-                if (File.Exists(Path.Combine(fullPath)))
-                    TryToAddContextMenuItemsForGitLFS(menu, change.Path);
+                TryToAddContextMenuItemsForGitLFS(menu, fullPath, change.Path);
             }
 
             var copyPath = new MenuItem();
@@ -407,7 +395,7 @@ namespace SourceGit.ViewModels
             copyFullPath.Icon = App.CreateMenuIcon("Icons.Copy");
             copyFullPath.Click += (_, e) =>
             {
-                App.CopyText(Native.OS.GetAbsPath(_repo.FullPath, change.Path));
+                App.CopyText(fullPath);
                 e.Handled = true;
             };
 
@@ -419,7 +407,7 @@ namespace SourceGit.ViewModels
         public ContextMenu CreateRevisionFileContextMenu(Models.Object file)
         {
             var menu = new ContextMenu();
-            var fullPath = Path.Combine(_repo.FullPath, file.Path);
+            var fullPath = Native.OS.GetAbsPath(_repo.FullPath, file.Path);
             var explore = new MenuItem();
             explore.Header = App.Text("RevealFile");
             explore.Icon = App.CreateMenuIcon("Icons.Explore");
@@ -433,12 +421,12 @@ namespace SourceGit.ViewModels
             var openWith = new MenuItem();
             openWith.Header = App.Text("OpenWith");
             openWith.Icon = App.CreateMenuIcon("Icons.OpenWith");
-            openWith.Click += (_, ev) =>
+            openWith.Click += async (_, ev) =>
             {
                 var fileName = Path.GetFileNameWithoutExtension(fullPath) ?? "";
                 var fileExt = Path.GetExtension(fullPath) ?? "";
                 var tmpFile = Path.Combine(Path.GetTempPath(), $"{fileName}~{_commit.SHA.Substring(0, 10)}{fileExt}");
-                Commands.SaveRevisionFile.Run(_repo.FullPath, _commit.SHA, file.Path, tmpFile);
+                await Task.Run(() => Commands.SaveRevisionFile.Run(_repo.FullPath, _commit.SHA, file.Path, tmpFile));
                 Native.OS.OpenWithDefaultEditor(tmpFile);
                 ev.Handled = true;
             };
@@ -459,8 +447,10 @@ namespace SourceGit.ViewModels
                     var selected = await storageProvider.OpenFolderPickerAsync(options);
                     if (selected.Count == 1)
                     {
-                        var saveTo = Path.Combine(selected[0].Path.LocalPath, Path.GetFileName(file.Path));
-                        Commands.SaveRevisionFile.Run(_repo.FullPath, _commit.SHA, file.Path, saveTo);
+                        var folder = selected[0];
+                        var folderPath = folder is { Path: { IsAbsoluteUri: true } path } ? path.LocalPath : folder.Path.ToString();
+                        var saveTo = Path.Combine(folderPath, Path.GetFileName(file.Path)!);
+                        await Task.Run(() => Commands.SaveRevisionFile.Run(_repo.FullPath, _commit.SHA, file.Path, saveTo));
                     }
                 }
                 catch (Exception e)
@@ -499,38 +489,34 @@ namespace SourceGit.ViewModels
             menu.Items.Add(blame);
             menu.Items.Add(new MenuItem() { Header = "-" });
 
-            var resetToThisRevision = new MenuItem();
-            resetToThisRevision.Header = App.Text("ChangeCM.CheckoutThisRevision");
-            resetToThisRevision.Icon = App.CreateMenuIcon("Icons.File.Checkout");
-            resetToThisRevision.IsEnabled = File.Exists(fullPath);
-            resetToThisRevision.Click += (_, ev) =>
+            if (!_repo.IsBare)
             {
-                var log = _repo.CreateLog($"Reset File to '{_commit.SHA}'");
-                new Commands.Checkout(_repo.FullPath).Use(log).FileWithRevision(file.Path, $"{_commit.SHA}");
-                log.Complete();
-                ev.Handled = true;
-            };
+                var resetToThisRevision = new MenuItem();
+                resetToThisRevision.Header = App.Text("ChangeCM.CheckoutThisRevision");
+                resetToThisRevision.Icon = App.CreateMenuIcon("Icons.File.Checkout");
+                resetToThisRevision.Click += async (_, ev) =>
+                {
+                    await ResetToThisRevision(file.Path);
+                    ev.Handled = true;
+                };
 
-            var resetToFirstParent = new MenuItem();
-            resetToFirstParent.Header = App.Text("ChangeCM.CheckoutFirstParentRevision");
-            resetToFirstParent.Icon = App.CreateMenuIcon("Icons.File.Checkout");
-            var fileInChanges = _changes.Find(x => x.Path == file.Path);
-            var fileIndex = fileInChanges?.Index;
-            resetToFirstParent.IsEnabled = _commit.Parents.Count > 0 && fileIndex != Models.ChangeState.Renamed;
-            resetToFirstParent.Click += (_, ev) =>
-            {
-                var log = _repo.CreateLog($"Reset File to '{_commit.SHA}~1'");
-                new Commands.Checkout(_repo.FullPath).Use(log).FileWithRevision(file.Path, $"{_commit.SHA}~1");
-                log.Complete();
-                ev.Handled = true;
-            };
+                var change = _changes.Find(x => x.Path == file.Path) ?? new Models.Change() { Index = Models.ChangeState.None, Path = file.Path };
+                var resetToFirstParent = new MenuItem();
+                resetToFirstParent.Header = App.Text("ChangeCM.CheckoutFirstParentRevision");
+                resetToFirstParent.Icon = App.CreateMenuIcon("Icons.File.Checkout");
+                resetToFirstParent.IsEnabled = _commit.Parents.Count > 0;
+                resetToFirstParent.Click += async (_, ev) =>
+                {
+                    await ResetToParentRevision(change);
+                    ev.Handled = true;
+                };
 
-            menu.Items.Add(resetToThisRevision);
-            menu.Items.Add(resetToFirstParent);
-            menu.Items.Add(new MenuItem() { Header = "-" });
+                menu.Items.Add(resetToThisRevision);
+                menu.Items.Add(resetToFirstParent);
+                menu.Items.Add(new MenuItem() { Header = "-" });
 
-            if (File.Exists(Path.Combine(fullPath)))
-                TryToAddContextMenuItemsForGitLFS(menu, file.Path);
+                TryToAddContextMenuItemsForGitLFS(menu, fullPath, file.Path);
+            }
 
             var copyPath = new MenuItem();
             copyPath.Header = App.Text("CopyPath");
@@ -546,7 +532,7 @@ namespace SourceGit.ViewModels
             copyFullPath.Icon = App.CreateMenuIcon("Icons.Copy");
             copyFullPath.Click += (_, e) =>
             {
-                App.CopyText(Native.OS.GetAbsPath(_repo.FullPath, file.Path));
+                App.CopyText(fullPath);
                 e.Handled = true;
             };
 
@@ -558,6 +544,7 @@ namespace SourceGit.ViewModels
         private void Refresh()
         {
             _changes = null;
+            _requestingRevisionFiles = false;
             _revisionFiles = null;
 
             SignInfo = null;
@@ -605,7 +592,7 @@ namespace SourceGit.ViewModels
 
             Task.Run(() =>
             {
-                var parent = _commit.Parents.Count == 0 ? "4b825dc642cb6eb9a060e54bf8d69288fbee4904" : _commit.Parents[0];
+                var parent = _commit.Parents.Count == 0 ? Models.Commit.EmptyTreeSHA1 : _commit.Parents[0];
                 var cmd = new Commands.CompareRevisions(_repo.FullPath, parent, _commit.SHA) { CancellationToken = token };
                 var changes = cmd.Result();
                 var visible = changes;
@@ -633,35 +620,42 @@ namespace SourceGit.ViewModels
             });
         }
 
-        private List<Models.InlineElement> ParseInlinesInMessage(string message)
+        private Models.InlineElementCollector ParseInlinesInMessage(string message)
         {
-            var inlines = new List<Models.InlineElement>();
+            var inlines = new Models.InlineElementCollector();
             if (_repo.Settings.IssueTrackerRules is { Count: > 0 } rules)
             {
                 foreach (var rule in rules)
                     rule.Matches(inlines, message);
             }
 
-            var matches = REG_SHA_FORMAT().Matches(message);
-            for (int i = 0; i < matches.Count; i++)
+            var urlMatches = REG_URL_FORMAT().Matches(message);
+            for (int i = 0; i < urlMatches.Count; i++)
             {
-                var match = matches[i];
+                var match = urlMatches[i];
                 if (!match.Success)
                     continue;
 
                 var start = match.Index;
                 var len = match.Length;
-                var intersect = false;
-                foreach (var link in inlines)
-                {
-                    if (link.Intersect(start, len))
-                    {
-                        intersect = true;
-                        break;
-                    }
-                }
+                if (inlines.Intersect(start, len) != null)
+                    continue;
 
-                if (intersect)
+                var url = message.Substring(start, len);
+                if (Uri.IsWellFormedUriString(url, UriKind.Absolute))
+                    inlines.Add(new Models.InlineElement(Models.InlineElementType.Link, start, len, url));
+            }
+
+            var shaMatches = REG_SHA_FORMAT().Matches(message);
+            for (int i = 0; i < shaMatches.Count; i++)
+            {
+                var match = shaMatches[i];
+                if (!match.Success)
+                    continue;
+
+                var start = match.Index;
+                var len = match.Length;
+                if (inlines.Intersect(start, len) != null)
                     continue;
 
                 var sha = match.Groups[1].Value;
@@ -670,9 +664,7 @@ namespace SourceGit.ViewModels
                     inlines.Add(new Models.InlineElement(Models.InlineElementType.CommitSHA, start, len, sha));
             }
 
-            if (inlines.Count > 0)
-                inlines.Sort((l, r) => l.Start - r.Start);
-
+            inlines.Sort();
             return inlines;
         }
 
@@ -698,8 +690,11 @@ namespace SourceGit.ViewModels
             }
         }
 
-        private void TryToAddContextMenuItemsForGitLFS(ContextMenu menu, string path)
+        private void TryToAddContextMenuItemsForGitLFS(ContextMenu menu, string fullPath, string path)
         {
+            if (_repo.Remotes.Count == 0 || !File.Exists(fullPath))
+                return;
+
             var lfsEnabled = new Commands.LFS(_repo.FullPath).IsEnabled();
             if (!lfsEnabled)
                 return;
@@ -711,7 +706,6 @@ namespace SourceGit.ViewModels
             var lfsLock = new MenuItem();
             lfsLock.Header = App.Text("GitLFS.Locks.Lock");
             lfsLock.Icon = App.CreateMenuIcon("Icons.Lock");
-            lfsLock.IsEnabled = _repo.Remotes.Count > 0;
             if (_repo.Remotes.Count == 1)
             {
                 lfsLock.Click += async (_, e) =>
@@ -750,7 +744,6 @@ namespace SourceGit.ViewModels
             var lfsUnlock = new MenuItem();
             lfsUnlock.Header = App.Text("GitLFS.Locks.Unlock");
             lfsUnlock.Icon = App.CreateMenuIcon("Icons.Unlock");
-            lfsUnlock.IsEnabled = _repo.Remotes.Count > 0;
             if (_repo.Remotes.Count == 1)
             {
                 lfsUnlock.Click += async (_, e) =>
@@ -796,16 +789,22 @@ namespace SourceGit.ViewModels
             {
                 if (_revisionFiles == null)
                 {
+                    if (_requestingRevisionFiles)
+                        return;
+
                     var sha = Commit.SHA;
+                    _requestingRevisionFiles = true;
 
                     Task.Run(() =>
                     {
                         var files = new Commands.QueryRevisionFileNames(_repo.FullPath, sha).Result();
                         Dispatcher.UIThread.Invoke(() =>
                         {
-                            if (sha == Commit.SHA)
+                            if (sha == Commit.SHA && _requestingRevisionFiles)
                             {
                                 _revisionFiles = files;
+                                _requestingRevisionFiles = false;
+
                                 if (!string.IsNullOrEmpty(_revisionFileSearchFilter))
                                     CalcRevisionFileSearchSuggestion();
                             }
@@ -840,17 +839,37 @@ namespace SourceGit.ViewModels
             RevisionFileSearchSuggestion = suggestion;
         }
 
+        private Task ResetToThisRevision(string path)
+        {
+            var log = _repo.CreateLog($"Reset File to '{_commit.SHA}'");
+
+            return Task.Run(() =>
+            {
+                new Commands.Checkout(_repo.FullPath).Use(log).FileWithRevision(path, $"{_commit.SHA}");
+                log.Complete();
+            });
+        }
+
+        private Task ResetToParentRevision(Models.Change change)
+        {
+            var log = _repo.CreateLog($"Reset File to '{_commit.SHA}~1'");
+
+            return Task.Run(() =>
+            {
+                if (change.Index == Models.ChangeState.Renamed)
+                    new Commands.Checkout(_repo.FullPath).Use(log).FileWithRevision(change.OriginalPath, $"{_commit.SHA}~1");
+
+                new Commands.Checkout(_repo.FullPath).Use(log).FileWithRevision(change.Path, $"{_commit.SHA}~1");
+                log.Complete();
+            });
+        }
+
+        [GeneratedRegex(@"\b(https?://|ftp://)[\w\d\._/\-~%@()+:?&=#!]*[\w\d/]")]
+        private static partial Regex REG_URL_FORMAT();
+
         [GeneratedRegex(@"\b([0-9a-fA-F]{6,40})\b")]
         private static partial Regex REG_SHA_FORMAT();
 
-        [GeneratedRegex(@"^version https://git-lfs.github.com/spec/v\d+\r?\noid sha256:([0-9a-f]+)\r?\nsize (\d+)[\r\n]*$")]
-        private static partial Regex REG_LFS_FORMAT();
-
-        private static readonly HashSet<string> IMG_EXTS = new HashSet<string>()
-        {
-            ".ico", ".bmp", ".jpg", ".png", ".jpeg", ".webp"
-        };
-
         private Repository _repo = null;
         private Models.Commit _commit = null;
         private Models.CommitFullMessage _fullMessage = null;
@@ -861,8 +880,10 @@ namespace SourceGit.ViewModels
         private List<Models.Change> _selectedChanges = null;
         private string _searchChangeFilter = string.Empty;
         private DiffContext _diffContext = null;
+        private string _viewRevisionFilePath = string.Empty;
         private object _viewRevisionFileContent = null;
         private CancellationTokenSource _cancellationSource = null;
+        private bool _requestingRevisionFiles = false;
         private List<string> _revisionFiles = null;
         private string _revisionFileSearchFilter = string.Empty;
         private List<string> _revisionFileSearchSuggestion = null;
