@@ -28,7 +28,14 @@ namespace SourceGit.ViewModels
         public bool CheckoutAfterCreated
         {
             get => _repo.Settings.CheckoutBranchOnCreateBranch;
-            set => _repo.Settings.CheckoutBranchOnCreateBranch = value;
+            set
+            {
+                if (_repo.Settings.CheckoutBranchOnCreateBranch != value)
+                {
+                    _repo.Settings.CheckoutBranchOnCreateBranch = value;
+                    OnPropertyChanged();
+                }
+            }
         }
 
         public bool IsBareRepository
@@ -36,6 +43,27 @@ namespace SourceGit.ViewModels
             get => _repo.IsBare;
         }
 
+        public bool AllowOverwrite
+        {
+            get => _allowOverwrite;
+            set
+            {
+                if (SetProperty(ref _allowOverwrite, value))
+                    ValidateProperty(_name, nameof(Name));
+            }
+        }
+
+        public bool IsRecurseSubmoduleVisible
+        {
+            get => _repo.Submodules.Count > 0;
+        }
+
+        public bool RecurseSubmodules
+        {
+            get => _repo.Settings.UpdateSubmodulesOnCheckoutBranch;
+            set => _repo.Settings.UpdateSubmodulesOnCheckoutBranch = value;
+        }
+
         public CreateBranch(Repository repo, Models.Branch branch)
         {
             _repo = repo;
@@ -70,18 +98,24 @@ namespace SourceGit.ViewModels
 
         public static ValidationResult ValidateBranchName(string name, ValidationContext ctx)
         {
-            var creator = ctx.ObjectInstance as CreateBranch;
-            if (creator == null)
-                return new ValidationResult("Missing runtime context to create branch!");
-
-            var fixedName = creator.FixName(name);
-            foreach (var b in creator._repo.Branches)
+            if (ctx.ObjectInstance is CreateBranch creator)
             {
-                if (b.FriendlyName == fixedName)
-                    return new ValidationResult("A branch with same name already exists!");
-            }
+                if (!creator._allowOverwrite)
+                {
+                    var fixedName = creator.FixName(name);
+                    foreach (var b in creator._repo.Branches)
+                    {
+                        if (b.FriendlyName == fixedName)
+                            return new ValidationResult("A branch with same name already exists!");
+                    }
+                }
 
-            return ValidationResult.Success;
+                return ValidationResult.Success;
+            }
+            else
+            {
+                return new ValidationResult("Missing runtime context to create branch!");
+            }
         }
 
         public override Task<bool> Sure()
@@ -92,20 +126,21 @@ namespace SourceGit.ViewModels
             var log = _repo.CreateLog($"Create Branch '{fixedName}'");
             Use(log);
 
+            var updateSubmodules = IsRecurseSubmoduleVisible && RecurseSubmodules;
             return Task.Run(() =>
             {
-                bool succ;
+                bool succ = false;
                 if (CheckoutAfterCreated && !_repo.IsBare)
                 {
-                    var changes = new Commands.CountLocalChangesWithoutUntracked(_repo.FullPath).Result();
                     var needPopStash = false;
-                    if (changes > 0)
+                    if (DiscardLocalChanges)
                     {
-                        if (DiscardLocalChanges)
-                        {
-                            Commands.Discard.All(_repo.FullPath, false, log);
-                        }
-                        else
+                        succ = new Commands.Checkout(_repo.FullPath).Use(log).Branch(fixedName, _baseOnRevision, true, _allowOverwrite);
+                    }
+                    else
+                    {
+                        var changes = new Commands.CountLocalChangesWithoutUntracked(_repo.FullPath).Result();
+                        if (changes > 0)
                         {
                             succ = new Commands.Stash(_repo.FullPath).Use(log).Push("CREATE_BRANCH_AUTO_STASH");
                             if (!succ)
@@ -117,15 +152,26 @@ namespace SourceGit.ViewModels
 
                             needPopStash = true;
                         }
+
+                        succ = new Commands.Checkout(_repo.FullPath).Use(log).Branch(fixedName, _baseOnRevision, false, _allowOverwrite);
                     }
 
-                    succ = new Commands.Checkout(_repo.FullPath).Use(log).Branch(fixedName, _baseOnRevision);
-                    if (needPopStash)
-                        new Commands.Stash(_repo.FullPath).Use(log).Pop("stash@{0}");
+                    if (succ)
+                    {
+                        if (updateSubmodules)
+                        {
+                            var submodules = new Commands.QueryUpdatableSubmodules(_repo.FullPath).Result();
+                            if (submodules.Count > 0)
+                                new Commands.Submodule(_repo.FullPath).Use(log).Update(submodules, true, true);
+                        }
+
+                        if (needPopStash)
+                            new Commands.Stash(_repo.FullPath).Use(log).Pop("stash@{0}");
+                    }
                 }
                 else
                 {
-                    succ = Commands.Branch.Create(_repo.FullPath, fixedName, _baseOnRevision, log);
+                    succ = Commands.Branch.Create(_repo.FullPath, fixedName, _baseOnRevision, _allowOverwrite, log);
                 }
 
                 log.Complete();
@@ -144,12 +190,17 @@ namespace SourceGit.ViewModels
 
                         if (_repo.HistoriesFilterMode == Models.FilterMode.Included)
                             _repo.SetBranchFilterMode(fake, Models.FilterMode.Included, true, false);
+
+                        ProgressDescription = "Waiting for branch updated...";
                     }
 
                     _repo.MarkBranchesDirtyManually();
                     _repo.SetWatcherEnabled(true);
                 });
 
+                if (CheckoutAfterCreated)
+                    Task.Delay(400).Wait();
+
                 return true;
             });
         }
@@ -166,5 +217,6 @@ namespace SourceGit.ViewModels
         private readonly Repository _repo = null;
         private string _name = null;
         private readonly string _baseOnRevision = null;
+        private bool _allowOverwrite = false;
     }
 }
