@@ -1,4 +1,4 @@
-﻿using System;
+using System;
 using System.Collections;
 using System.Collections.Generic;
 using System.IO;
@@ -12,7 +12,7 @@ using CommunityToolkit.Mvvm.ComponentModel;
 
 namespace SourceGit.ViewModels
 {
-    public class Histories : ObservableObject
+    public class Histories : ObservableObject, IDisposable
     {
         public Repository Repo
         {
@@ -57,7 +57,7 @@ namespace SourceGit.ViewModels
             private set => SetProperty(ref _navigationId, value);
         }
 
-        public object DetailContext
+        public IDisposable DetailContext
         {
             get => _detailContext;
             set => SetProperty(ref _detailContext, value);
@@ -98,23 +98,13 @@ namespace SourceGit.ViewModels
             _repo = repo;
         }
 
-        public void Cleanup()
+        public void Dispose()
         {
-            Commits = new List<Models.Commit>();
-
+            Commits = [];
             _repo = null;
             _graph = null;
             _autoSelectedCommit = null;
-
-            if (_detailContext is CommitDetail cd)
-            {
-                cd.Cleanup();
-            }
-            else if (_detailContext is RevisionCompare rc)
-            {
-                rc.Cleanup();
-            }
-
+            _detailContext?.Dispose();
             _detailContext = null;
         }
 
@@ -220,7 +210,7 @@ namespace SourceGit.ViewModels
             else
             {
                 _repo.SelectedSearchedCommit = null;
-                DetailContext = commits.Count;
+                DetailContext = new Models.Count(commits.Count);
             }
         }
 
@@ -343,10 +333,12 @@ namespace SourceGit.ViewModels
                         {
                             log = _repo.CreateLog("Save as Patch");
 
+                            var folder = picker[0];
+                            var folderPath = folder is { Path: { IsAbsoluteUri: true } path } ? path.LocalPath : folder?.Path.ToString();
                             var succ = false;
                             for (var i = 0; i < selected.Count; i++)
                             {
-                                var saveTo = GetPatchFileName(picker[0].Path.LocalPath, selected[i], i);
+                                var saveTo = GetPatchFileName(folderPath, selected[i], i);
                                 succ = await Task.Run(() => new Commands.FormatPatch(_repo.FullPath, selected[i].SHA, saveTo).Use(log).Exec());
                                 if (!succ)
                                     break;
@@ -387,7 +379,7 @@ namespace SourceGit.ViewModels
                 {
                     var builder = new StringBuilder();
                     foreach (var c in selected)
-                        builder.AppendLine($"{c.SHA.Substring(0, 10)} - {c.Subject}");
+                        builder.AppendLine($"{c.SHA.AsSpan(0, 10)} - {c.Subject}");
 
                     App.CopyText(builder.ToString());
                     e.Handled = true;
@@ -701,8 +693,10 @@ namespace SourceGit.ViewModels
                     {
                         log = _repo.CreateLog("Save as Patch");
 
-                        var saveTo = GetPatchFileName(selected[0].Path.LocalPath, commit);
-                        var succ = new Commands.FormatPatch(_repo.FullPath, commit.SHA, saveTo).Use(log).Exec();
+                        var folder = selected[0];
+                        var folderPath = folder is { Path: { IsAbsoluteUri: true } path } ? path.LocalPath : folder?.Path.ToString();
+                        var saveTo = GetPatchFileName(folderPath, commit);
+                        var succ = await Task.Run(() => new Commands.FormatPatch(_repo.FullPath, commit.SHA, saveTo).Use(log).Exec());
                         if (succ)
                             App.SendNotification(_repo.FullPath, App.Text("SaveAsPatchSuccess"));
                     }
@@ -780,7 +774,7 @@ namespace SourceGit.ViewModels
             copyInfo.Icon = App.CreateMenuIcon("Icons.Info");
             copyInfo.Click += (_, e) =>
             {
-                App.CopyText($"{commit.SHA.Substring(0, 10)} - {commit.Subject}");
+                App.CopyText($"{commit.SHA.AsSpan(0, 10)} - {commit.Subject}");
                 e.Handled = true;
             };
 
@@ -815,116 +809,17 @@ namespace SourceGit.ViewModels
             return menu;
         }
 
-        private Models.FilterMode GetFilterMode(string pattern)
-        {
-            foreach (var filter in _repo.Settings.HistoriesFilters)
-            {
-                if (filter.Pattern.Equals(pattern, StringComparison.Ordinal))
-                    return filter.Mode;
-            }
-
-            return Models.FilterMode.None;
-        }
-
-        private void FillBranchVisibilityMenu(MenuItem submenu, Models.Branch branch)
-        {
-            var visibility = new MenuItem();
-            visibility.Icon = App.CreateMenuIcon("Icons.Eye");
-            visibility.Header = App.Text("Repository.FilterCommits");
-
-            var exclude = new MenuItem();
-            exclude.Icon = App.CreateMenuIcon("Icons.EyeClose");
-            exclude.Header = App.Text("Repository.FilterCommits.Exclude");
-            exclude.Click += (_, e) =>
-            {
-                _repo.SetBranchFilterMode(branch, Models.FilterMode.Excluded, false, true);
-                e.Handled = true;
-            };
-
-            var filterMode = GetFilterMode(branch.FullName);
-            if (filterMode == Models.FilterMode.None)
-            {
-                var include = new MenuItem();
-                include.Icon = App.CreateMenuIcon("Icons.Filter");
-                include.Header = App.Text("Repository.FilterCommits.Include");
-                include.Click += (_, e) =>
-                {
-                    _repo.SetBranchFilterMode(branch, Models.FilterMode.Included, false, true);
-                    e.Handled = true;
-                };
-                visibility.Items.Add(include);
-                visibility.Items.Add(exclude);
-            }
-            else
-            {
-                var unset = new MenuItem();
-                unset.Header = App.Text("Repository.FilterCommits.Default");
-                unset.Click += (_, e) =>
-                {
-                    _repo.SetBranchFilterMode(branch, Models.FilterMode.None, false, true);
-                    e.Handled = true;
-                };
-                visibility.Items.Add(exclude);
-                visibility.Items.Add(unset);
-            }
-
-            submenu.Items.Add(visibility);
-            submenu.Items.Add(new MenuItem() { Header = "-" });
-        }
-
-        private void FillTagVisibilityMenu(MenuItem submenu, Models.Tag tag)
-        {
-            var visibility = new MenuItem();
-            visibility.Icon = App.CreateMenuIcon("Icons.Eye");
-            visibility.Header = App.Text("Repository.FilterCommits");
-
-            var exclude = new MenuItem();
-            exclude.Icon = App.CreateMenuIcon("Icons.EyeClose");
-            exclude.Header = App.Text("Repository.FilterCommits.Exclude");
-            exclude.Click += (_, e) =>
-            {
-                _repo.SetTagFilterMode(tag, Models.FilterMode.Excluded);
-                e.Handled = true;
-            };
-
-            var filterMode = GetFilterMode(tag.Name);
-            if (filterMode == Models.FilterMode.None)
-            {
-                var include = new MenuItem();
-                include.Icon = App.CreateMenuIcon("Icons.Filter");
-                include.Header = App.Text("Repository.FilterCommits.Include");
-                include.Click += (_, e) =>
-                {
-                    _repo.SetTagFilterMode(tag, Models.FilterMode.Included);
-                    e.Handled = true;
-                };
-                visibility.Items.Add(include);
-                visibility.Items.Add(exclude);
-            }
-            else
-            {
-                var unset = new MenuItem();
-                unset.Header = App.Text("Repository.FilterCommits.Default");
-                unset.Click += (_, e) =>
-                {
-                    _repo.SetTagFilterMode(tag, Models.FilterMode.None);
-                    e.Handled = true;
-                };
-                visibility.Items.Add(exclude);
-                visibility.Items.Add(unset);
-            }
-
-            submenu.Items.Add(visibility);
-            submenu.Items.Add(new MenuItem() { Header = "-" });
-        }
-
         private void FillCurrentBranchMenu(ContextMenu menu, Models.Branch current)
         {
             var submenu = new MenuItem();
             submenu.Icon = App.CreateMenuIcon("Icons.Branch");
             submenu.Header = current.Name;
 
-            FillBranchVisibilityMenu(submenu, current);
+            var visibility = new MenuItem();
+            visibility.Classes.Add("filter_mode_switcher");
+            visibility.Header = new FilterModeInGraph(_repo, current);
+            submenu.Items.Add(visibility);
+            submenu.Items.Add(new MenuItem() { Header = "-" });
 
             if (!string.IsNullOrEmpty(current.Upstream))
             {
@@ -985,8 +880,8 @@ namespace SourceGit.ViewModels
 
             if (!_repo.IsBare)
             {
-                var detect = Commands.GitFlow.DetectType(_repo.FullPath, _repo.Branches, current.Name);
-                if (detect.IsGitFlowBranch)
+                var type = _repo.GetGitFlowType(current);
+                if (type != Models.GitFlowBranchType.None)
                 {
                     var finish = new MenuItem();
                     finish.Header = App.Text("BranchCM.Finish", current.Name);
@@ -994,7 +889,7 @@ namespace SourceGit.ViewModels
                     finish.Click += (_, e) =>
                     {
                         if (_repo.CanCreatePopup())
-                            _repo.ShowPopup(new GitFlowFinish(_repo, current, detect.Type, detect.Prefix));
+                            _repo.ShowPopup(new GitFlowFinish(_repo, current, type));
                         e.Handled = true;
                     };
                     submenu.Items.Add(finish);
@@ -1021,7 +916,11 @@ namespace SourceGit.ViewModels
             submenu.Icon = App.CreateMenuIcon("Icons.Branch");
             submenu.Header = branch.Name;
 
-            FillBranchVisibilityMenu(submenu, branch);
+            var visibility = new MenuItem();
+            visibility.Classes.Add("filter_mode_switcher");
+            visibility.Header = new FilterModeInGraph(_repo, branch);
+            submenu.Items.Add(visibility);
+            submenu.Items.Add(new MenuItem() { Header = "-" });
 
             if (!_repo.IsBare)
             {
@@ -1073,8 +972,8 @@ namespace SourceGit.ViewModels
 
             if (!_repo.IsBare)
             {
-                var detect = Commands.GitFlow.DetectType(_repo.FullPath, _repo.Branches, branch.Name);
-                if (detect.IsGitFlowBranch)
+                var type = _repo.GetGitFlowType(branch);
+                if (type != Models.GitFlowBranchType.None)
                 {
                     var finish = new MenuItem();
                     finish.Header = App.Text("BranchCM.Finish", branch.Name);
@@ -1082,7 +981,7 @@ namespace SourceGit.ViewModels
                     finish.Click += (_, e) =>
                     {
                         if (_repo.CanCreatePopup())
-                            _repo.ShowPopup(new GitFlowFinish(_repo, branch, detect.Type, detect.Prefix));
+                            _repo.ShowPopup(new GitFlowFinish(_repo, branch, type));
                         e.Handled = true;
                     };
                     submenu.Items.Add(finish);
@@ -1111,7 +1010,11 @@ namespace SourceGit.ViewModels
             submenu.Icon = App.CreateMenuIcon("Icons.Branch");
             submenu.Header = name;
 
-            FillBranchVisibilityMenu(submenu, branch);
+            var visibility = new MenuItem();
+            visibility.Classes.Add("filter_mode_switcher");
+            visibility.Header = new FilterModeInGraph(_repo, branch);
+            submenu.Items.Add(visibility);
+            submenu.Items.Add(new MenuItem() { Header = "-" });
 
             var checkout = new MenuItem();
             checkout.Header = App.Text("BranchCM.Checkout", name);
@@ -1168,7 +1071,11 @@ namespace SourceGit.ViewModels
             submenu.Icon = App.CreateMenuIcon("Icons.Tag");
             submenu.MinWidth = 200;
 
-            FillTagVisibilityMenu(submenu, tag);
+            var visibility = new MenuItem();
+            visibility.Classes.Add("filter_mode_switcher");
+            visibility.Header = new FilterModeInGraph(_repo, tag);
+            submenu.Items.Add(visibility);
+            submenu.Items.Add(new MenuItem() { Header = "-" });
 
             var push = new MenuItem();
             push.Header = App.Text("TagCM.Push", tag.Name);
@@ -1256,7 +1163,7 @@ namespace SourceGit.ViewModels
         private Models.CommitGraph _graph = null;
         private Models.Commit _autoSelectedCommit = null;
         private long _navigationId = 0;
-        private object _detailContext = null;
+        private IDisposable _detailContext = null;
 
         private Models.Bisect _bisect = null;
 
