@@ -1,4 +1,4 @@
-﻿using System;
+using System;
 using System.IO;
 
 using Avalonia.Collections;
@@ -44,6 +44,12 @@ namespace SourceGit.ViewModels
             }
         }
 
+        public IDisposable Switcher
+        {
+            get => _switcher;
+            private set => SetProperty(ref _switcher, value);
+        }
+
         public Launcher(string startupRepo)
         {
             _ignoreIndexChange = true;
@@ -130,6 +136,85 @@ namespace SourceGit.ViewModels
             _ignoreIndexChange = false;
         }
 
+        public void OpenWorkspaceSwitcher()
+        {
+            Switcher = new WorkspaceSwitcher(this);
+        }
+
+        public void OpenTabSwitcher()
+        {
+            Switcher = new LauncherPageSwitcher(this);
+        }
+
+        public void CancelSwitcher()
+        {
+            Switcher?.Dispose();
+            Switcher = null;
+        }
+
+        public void SwitchWorkspace(Workspace to)
+        {
+            if (to == null || to.IsActive)
+                return;
+
+            foreach (var one in Pages)
+            {
+                if (!one.CanCreatePopup() || one.Data is Repository { IsAutoFetching: true })
+                {
+                    App.RaiseException(null, "You have unfinished task(s) in opened pages. Please wait!!!");
+                    return;
+                }
+            }
+
+            _ignoreIndexChange = true;
+
+            var pref = Preferences.Instance;
+            foreach (var w in pref.Workspaces)
+                w.IsActive = false;
+
+            ActiveWorkspace = to;
+            to.IsActive = true;
+
+            foreach (var one in Pages)
+                CloseRepositoryInTab(one, false);
+
+            Pages.Clear();
+            AddNewTab();
+
+            var repos = to.Repositories.ToArray();
+            foreach (var repo in repos)
+            {
+                var node = pref.FindNode(repo);
+                if (node == null)
+                {
+                    node = new RepositoryNode()
+                    {
+                        Id = repo,
+                        Name = Path.GetFileName(repo),
+                        Bookmark = 0,
+                        IsRepository = true,
+                    };
+                }
+
+                OpenRepositoryInTab(node, null);
+            }
+
+            var activeIdx = to.ActiveIdx;
+            if (activeIdx >= 0 && activeIdx < Pages.Count)
+            {
+                ActivePage = Pages[activeIdx];
+            }
+            else
+            {
+                ActivePage = Pages[0];
+                to.ActiveIdx = 0;
+            }
+
+            _ignoreIndexChange = false;
+            Preferences.Instance.Save();
+            GC.Collect();
+        }
+
         public void AddNewTab()
         {
             var page = new LauncherPage();
@@ -275,7 +360,7 @@ namespace SourceGit.ViewModels
 
             if (!Path.Exists(node.Id))
             {
-                App.RaiseException(node.Id, "Repository does NOT exists any more. Please remove it.");
+                App.RaiseException(node.Id, "Repository does NOT exist any more. Please remove it.");
                 return;
             }
 
@@ -336,7 +421,7 @@ namespace SourceGit.ViewModels
 
             foreach (var page in Pages)
             {
-                var id = page.Node.Id.Replace("\\", "/");
+                var id = page.Node.Id.Replace('\\', '/').TrimEnd('/');
                 if (id == pageId)
                 {
                     page.Notifications.Add(notification);
@@ -494,66 +579,6 @@ namespace SourceGit.ViewModels
             return new Commands.QueryGitDir(repo).Result();
         }
 
-        private void SwitchWorkspace(Workspace to)
-        {
-            foreach (var one in Pages)
-            {
-                if (!one.CanCreatePopup() || one.Data is Repository { IsAutoFetching: true })
-                {
-                    App.RaiseException(null, "You have unfinished task(s) in opened pages. Please wait!!!");
-                    return;
-                }
-            }
-
-            _ignoreIndexChange = true;
-
-            var pref = Preferences.Instance;
-            foreach (var w in pref.Workspaces)
-                w.IsActive = false;
-
-            ActiveWorkspace = to;
-            to.IsActive = true;
-
-            foreach (var one in Pages)
-                CloseRepositoryInTab(one, false);
-
-            Pages.Clear();
-            AddNewTab();
-
-            var repos = to.Repositories.ToArray();
-            foreach (var repo in repos)
-            {
-                var node = pref.FindNode(repo);
-                if (node == null)
-                {
-                    node = new RepositoryNode()
-                    {
-                        Id = repo,
-                        Name = Path.GetFileName(repo),
-                        Bookmark = 0,
-                        IsRepository = true,
-                    };
-                }
-
-                OpenRepositoryInTab(node, null);
-            }
-
-            var activeIdx = to.ActiveIdx;
-            if (activeIdx >= 0 && activeIdx < Pages.Count)
-            {
-                ActivePage = Pages[activeIdx];
-            }
-            else
-            {
-                ActivePage = Pages[0];
-                to.ActiveIdx = 0;
-            }
-
-            _ignoreIndexChange = false;
-            Preferences.Instance.Save();
-            GC.Collect();
-        }
-
         private void CloseRepositoryInTab(LauncherPage page, bool removeFromWorkspace = true)
         {
             if (page.Data is Repository repo)
@@ -573,7 +598,7 @@ namespace SourceGit.ViewModels
                 return;
 
             var workspace = _activeWorkspace.Name;
-            if (_activePage is { Data: Repository repo })
+            if (_activePage is { Data: Repository })
             {
                 var node = _activePage.Node;
                 var name = node.Name;
@@ -584,7 +609,7 @@ namespace SourceGit.ViewModels
                     var home = Environment.GetFolderPath(Environment.SpecialFolder.UserProfile);
                     var prefixLen = home.EndsWith('/') ? home.Length - 1 : home.Length;
                     if (path.StartsWith(home, StringComparison.Ordinal))
-                        path = "~" + path.Substring(prefixLen);
+                        path = $"~{path.AsSpan(prefixLen)}";
                 }
 
                 Title = $"[{workspace}] {name} ({path})";
@@ -599,5 +624,6 @@ namespace SourceGit.ViewModels
         private LauncherPage _activePage = null;
         private bool _ignoreIndexChange = false;
         private string _title = string.Empty;
+        private IDisposable _switcher = null;
     }
 }
