@@ -91,8 +91,8 @@ namespace SourceGit.ViewModels
 
         public bool UseSystemWindowFrame
         {
-            get => _useSystemWindowFrame;
-            set => SetProperty(ref _useSystemWindowFrame, value);
+            get => Native.OS.UseSystemWindowFrame;
+            set => Native.OS.UseSystemWindowFrame = value;
         }
 
         public double DefaultFontSize
@@ -178,9 +178,9 @@ namespace SourceGit.ViewModels
 
         public bool ShowTagsAsTree
         {
-            get => _showTagsAsTree;
-            set => SetProperty(ref _showTagsAsTree, value);
-        }
+            get;
+            set;
+        } = false;
 
         public bool ShowTagsInGraph
         {
@@ -188,6 +188,12 @@ namespace SourceGit.ViewModels
             set => SetProperty(ref _showTagsInGraph, value);
         }
 
+        public bool ShowSubmodulesAsTree
+        {
+            get;
+            set;
+        } = false;
+
         public bool UseTwoColumnsLayoutInHistories
         {
             get => _useTwoColumnsLayoutInHistories;
@@ -255,6 +261,12 @@ namespace SourceGit.ViewModels
             set => SetProperty(ref _useBlockNavigationInDiffView, value);
         }
 
+        public int LFSImageActiveIdx
+        {
+            get => _lfsImageActiveIdx;
+            set => SetProperty(ref _lfsImageActiveIdx, value);
+        }
+
         public Models.ChangeViewMode UnstagedChangeViewMode
         {
             get => _unstagedChangeViewMode;
@@ -424,16 +436,21 @@ namespace SourceGit.ViewModels
         {
             var collection = to == null ? RepositoryNodes : to.SubNodes;
             collection.Add(node);
-            collection.Sort((l, r) =>
+            SortNodes(collection);
+
+            if (save)
+                Save();
+        }
+
+        public void SortNodes(List<RepositoryNode> collection)
+        {
+            collection?.Sort((l, r) =>
             {
                 if (l.IsRepository != r.IsRepository)
                     return l.IsRepository ? 1 : -1;
 
                 return string.Compare(l.Name, r.Name, StringComparison.Ordinal);
             });
-
-            if (save)
-                Save();
         }
 
         public RepositoryNode FindNode(string id)
@@ -441,11 +458,9 @@ namespace SourceGit.ViewModels
             return FindNodeRecursive(id, RepositoryNodes);
         }
 
-        public RepositoryNode FindOrAddNodeByRepositoryPath(string repo, RepositoryNode parent, bool shouldMoveNode)
+        public RepositoryNode FindOrAddNodeByRepositoryPath(string repo, RepositoryNode parent, bool shouldMoveNode, bool save = true)
         {
-            var normalized = repo.Replace('\\', '/');
-            if (normalized.EndsWith("/"))
-                normalized = normalized.TrimEnd('/');
+            var normalized = repo.Replace('\\', '/').TrimEnd('/');
 
             var node = FindNodeRecursive(normalized, RepositoryNodes);
             if (node == null)
@@ -458,11 +473,11 @@ namespace SourceGit.ViewModels
                     IsRepository = true,
                 };
 
-                AddNode(node, parent, true);
+                AddNode(node, parent, save);
             }
             else if (shouldMoveNode)
             {
-                MoveNode(node, parent, true);
+                MoveNode(node, parent, save);
             }
 
             return node;
@@ -493,22 +508,13 @@ namespace SourceGit.ViewModels
         public void SortByRenamedNode(RepositoryNode node)
         {
             var container = FindNodeContainer(node, RepositoryNodes);
-            container?.Sort((l, r) =>
-            {
-                if (l.IsRepository != r.IsRepository)
-                    return l.IsRepository ? 1 : -1;
-
-                return string.Compare(l.Name, r.Name, StringComparison.Ordinal);
-            });
-
+            SortNodes(container);
             Save();
         }
 
         public void AutoRemoveInvalidNode()
         {
-            var changed = RemoveInvalidRepositoriesRecursive(RepositoryNodes);
-            if (changed)
-                Save();
+            RemoveInvalidRepositoriesRecursive(RepositoryNodes);
         }
 
         public void Save()
@@ -578,6 +584,13 @@ namespace SourceGit.ViewModels
             }
         }
 
+        private void SortNodesRecursive(List<RepositoryNode> collection)
+        {
+            SortNodes(collection);
+            foreach (var node in collection)
+                SortNodesRecursive(node.SubNodes);
+        }
+
         private RepositoryNode FindNodeRecursive(string id, List<RepositoryNode> collection)
         {
             foreach (var node in collection)
@@ -655,8 +668,7 @@ namespace SourceGit.ViewModels
         private string _themeOverrides = string.Empty;
         private string _defaultFontFamily = string.Empty;
         private string _monospaceFontFamily = string.Empty;
-        private bool _onlyUseMonoFontInEditor = false;
-        private bool _useSystemWindowFrame = false;
+        private bool _onlyUseMonoFontInEditor = true;
         private double _defaultFontSize = 13;
         private double _editorFontSize = 13;
         private int _editorTabWidth = 4;
@@ -672,7 +684,6 @@ namespace SourceGit.ViewModels
         private double _lastCheckUpdateTime = 0;
         private string _ignoreUpdateTag = string.Empty;
 
-        private bool _showTagsAsTree = false;
         private bool _showTagsInGraph = true;
         private bool _useTwoColumnsLayoutInHistories = false;
         private bool _displayTimeAsPeriodInHistories = false;
@@ -683,6 +694,7 @@ namespace SourceGit.ViewModels
         private bool _showHiddenSymbolsInDiffView = false;
         private bool _useFullTextDiff = false;
         private bool _useBlockNavigationInDiffView = false;
+        private int _lfsImageActiveIdx = 0;
 
         private Models.ChangeViewMode _unstagedChangeViewMode = Models.ChangeViewMode.List;
         private Models.ChangeViewMode _stagedChangeViewMode = Models.ChangeViewMode.List;
