@@ -34,6 +34,7 @@ namespace SourceGit.ViewModels
 
             return Task.Run(() =>
             {
+                var signOff = _repo.Settings.EnableSignOffForCommit;
                 var autoStashed = false;
                 var succ = false;
 
@@ -52,7 +53,7 @@ namespace SourceGit.ViewModels
 
                 succ = new Commands.Reset(_repo.FullPath, Target.SHA, "--soft").Use(log).Exec();
                 if (succ)
-                    succ = new Commands.Commit(_repo.FullPath, _message, true, _repo.Settings.EnableSignOffForCommit).Use(log).Run();
+                    succ = new Commands.Commit(_repo.FullPath, _message, signOff, true, false).Use(log).Run();
 
                 if (succ && autoStashed)
                     new Commands.Stash(_repo.FullPath).Use(log).Pop("stash@{0}");
