@@ -11,7 +11,7 @@ using CommunityToolkit.Mvvm.ComponentModel;
 
 namespace SourceGit.ViewModels
 {
-    public class WorkingCopy : ObservableObject
+    public class WorkingCopy : ObservableObject, IDisposable
     {
         public bool IncludeUntracked
         {
@@ -91,6 +91,7 @@ namespace SourceGit.ViewModels
                     else
                     {
                         CommitMessage = string.Empty;
+                        ResetAuthor = false;
                     }
 
                     Staged = GetStagedChanges();
@@ -100,6 +101,12 @@ namespace SourceGit.ViewModels
             }
         }
 
+        public bool ResetAuthor
+        {
+            get => _resetAuthor;
+            set => SetProperty(ref _resetAuthor, value);
+        }
+
         public string Filter
         {
             get => _filter;
@@ -210,7 +217,7 @@ namespace SourceGit.ViewModels
             _repo = repo;
         }
 
-        public void Cleanup()
+        public void Dispose()
         {
             _repo = null;
             _inProgressContext = null;
@@ -244,7 +251,7 @@ namespace SourceGit.ViewModels
                 // Just force refresh selected changes.
                 Dispatcher.UIThread.Invoke(() =>
                 {
-                    HasUnsolvedConflicts = _cached.Find(x => x.IsConflict) != null;
+                    HasUnsolvedConflicts = _cached.Find(x => x.IsConflicted) != null;
 
                     UpdateDetail();
                     UpdateInProgressState();
@@ -276,7 +283,7 @@ namespace SourceGit.ViewModels
                 if (c.WorkTree != Models.ChangeState.None)
                 {
                     unstaged.Add(c);
-                    hasConflict |= c.IsConflict;
+                    hasConflict |= c.IsConflicted;
                 }
             }
 
@@ -378,10 +385,12 @@ namespace SourceGit.ViewModels
 
             foreach (var change in changes)
             {
-                if (!change.IsConflict)
+                if (!change.IsConflicted)
                     continue;
 
-                if (change.WorkTree == Models.ChangeState.Deleted)
+                if (change.ConflictReason == Models.ConflictReason.BothDeleted ||
+                    change.ConflictReason == Models.ConflictReason.DeletedByThem ||
+                    change.ConflictReason == Models.ConflictReason.AddedByUs)
                 {
                     var fullpath = Path.Combine(_repo.FullPath, change.Path);
                     if (File.Exists(fullpath))
@@ -403,7 +412,12 @@ namespace SourceGit.ViewModels
             }
 
             if (needStage.Count > 0)
-                await Task.Run(() => new Commands.Add(_repo.FullPath, needStage).Use(log).Exec());
+            {
+                var pathSpecFile = Path.GetTempFileName();
+                await File.WriteAllLinesAsync(pathSpecFile, needStage);
+                await Task.Run(() => new Commands.Add(_repo.FullPath, pathSpecFile).Use(log).Exec());
+                File.Delete(pathSpecFile);
+            }
 
             log.Complete();
             _repo.MarkWorkingCopyDirtyManually();
@@ -420,10 +434,12 @@ namespace SourceGit.ViewModels
 
             foreach (var change in changes)
             {
-                if (!change.IsConflict)
+                if (!change.IsConflicted)
                     continue;
 
-                if (change.Index == Models.ChangeState.Deleted)
+                if (change.ConflictReason == Models.ConflictReason.BothDeleted ||
+                    change.ConflictReason == Models.ConflictReason.DeletedByUs ||
+                    change.ConflictReason == Models.ConflictReason.AddedByThem)
                 {
                     var fullpath = Path.Combine(_repo.FullPath, change.Path);
                     if (File.Exists(fullpath))
@@ -445,7 +461,12 @@ namespace SourceGit.ViewModels
             }
 
             if (needStage.Count > 0)
-                await Task.Run(() => new Commands.Add(_repo.FullPath, needStage).Use(log).Exec());
+            {
+                var pathSpecFile = Path.GetTempFileName();
+                await File.WriteAllLinesAsync(pathSpecFile, needStage);
+                await Task.Run(() => new Commands.Add(_repo.FullPath, pathSpecFile).Use(log).Exec());
+                File.Delete(pathSpecFile);
+            }
 
             log.Complete();
             _repo.MarkWorkingCopyDirtyManually();
@@ -456,7 +477,7 @@ namespace SourceGit.ViewModels
         {
             var toolType = Preferences.Instance.ExternalMergeToolType;
             var toolPath = Preferences.Instance.ExternalMergeToolPath;
-            var file = change?.Path; // NOTE: With no <file> arg, mergetool runs on on every file with merge conflicts!
+            var file = change?.Path; // NOTE: With no <file> arg, mergetool runs on every file with merge conflicts!
             await Task.Run(() => Commands.MergeTool.OpenForMerge(_repo.FullPath, toolType, toolPath, file));
         }
 
@@ -547,17 +568,17 @@ namespace SourceGit.ViewModels
 
         public void Commit()
         {
-            DoCommit(false, false, false);
+            DoCommit(false, false);
         }
 
         public void CommitWithAutoStage()
         {
-            DoCommit(true, false, false);
+            DoCommit(true, false);
         }
 
         public void CommitWithPush()
         {
-            DoCommit(false, true, false);
+            DoCommit(false, true);
         }
 
         public ContextMenu CreateContextMenuForUnstagedChanges()
@@ -594,7 +615,7 @@ namespace SourceGit.ViewModels
                 menu.Items.Add(openWith);
                 menu.Items.Add(new MenuItem() { Header = "-" });
 
-                if (change.IsConflict)
+                if (change.IsConflicted)
                 {
                     var useTheirs = new MenuItem();
                     useTheirs.Icon = App.CreateMenuIcon("Icons.Incoming");
@@ -635,7 +656,7 @@ namespace SourceGit.ViewModels
                     }
                     else if (_inProgressContext is RevertInProgress revert)
                     {
-                        useTheirs.Header = App.Text("FileCM.ResolveUsing", $"{revert.Head.SHA.AsSpan().Slice(0, 10)} (revert)");
+                        useTheirs.Header = App.Text("FileCM.ResolveUsing", $"{revert.Head.SHA.AsSpan(0, 10)} (revert)");
                         useMine.Header = App.Text("FileCM.ResolveUsing", _repo.CurrentBranch.Name);
                     }
                     else if (_inProgressContext is MergeInProgress merge)
@@ -759,7 +780,7 @@ namespace SourceGit.ViewModels
                         byParentFolder.IsVisible = !isRooted;
                         byParentFolder.Click += (_, e) =>
                         {
-                            var dir = Path.GetDirectoryName(change.Path).Replace("\\", "/");
+                            var dir = Path.GetDirectoryName(change.Path)!.Replace('\\', '/').TrimEnd('/');
                             Commands.GitIgnore.Add(_repo.FullPath, dir + "/");
                             e.Handled = true;
                         };
@@ -781,7 +802,7 @@ namespace SourceGit.ViewModels
                             byExtensionInSameFolder.IsVisible = !isRooted;
                             byExtensionInSameFolder.Click += (_, e) =>
                             {
-                                var dir = Path.GetDirectoryName(change.Path).Replace("\\", "/");
+                                var dir = Path.GetDirectoryName(change.Path)!.Replace('\\', '/').TrimEnd('/');
                                 Commands.GitIgnore.Add(_repo.FullPath, $"{dir}/*{extension}");
                                 e.Handled = true;
                             };
@@ -949,7 +970,7 @@ namespace SourceGit.ViewModels
                 var hasNonConflicts = false;
                 foreach (var change in _selectedUnstaged)
                 {
-                    if (change.IsConflict)
+                    if (change.IsConflicted)
                         hasConflicts = true;
                     else
                         hasNonConflicts = true;
@@ -993,7 +1014,7 @@ namespace SourceGit.ViewModels
                     }
                     else if (_inProgressContext is RevertInProgress revert)
                     {
-                        useTheirs.Header = App.Text("FileCM.ResolveUsing", $"{revert.Head.SHA.AsSpan().Slice(0, 10)} (revert)");
+                        useTheirs.Header = App.Text("FileCM.ResolveUsing", $"{revert.Head.SHA.AsSpan(0, 10)} (revert)");
                         useMine.Header = App.Text("FileCM.ResolveUsing", _repo.CurrentBranch.Name);
                     }
                     else if (_inProgressContext is MergeInProgress merge)
@@ -1078,7 +1099,7 @@ namespace SourceGit.ViewModels
             var menu = new ContextMenu();
 
             var ai = null as MenuItem;
-            var services = _repo.GetPreferedOpenAIServices();
+            var services = _repo.GetPreferredOpenAIServices();
             if (services.Count > 0)
             {
                 ai = new MenuItem();
@@ -1417,7 +1438,7 @@ namespace SourceGit.ViewModels
                         var home = Environment.GetFolderPath(Environment.SpecialFolder.UserProfile);
                         var prefixLen = home.EndsWith('/') ? home.Length - 1 : home.Length;
                         if (gitTemplate.StartsWith(home, StringComparison.Ordinal))
-                            friendlyName = $"~{gitTemplate.AsSpan().Slice(prefixLen)}";
+                            friendlyName = $"~{gitTemplate.AsSpan(prefixLen)}";
                     }
 
                     var gitTemplateItem = new MenuItem();
@@ -1476,7 +1497,7 @@ namespace SourceGit.ViewModels
                 return null;
             }
 
-            var services = _repo.GetPreferedOpenAIServices();
+            var services = _repo.GetPreferredOpenAIServices();
             if (services.Count == 0)
             {
                 App.RaiseException(_repo.FullPath, "Bad configuration for OpenAI");
@@ -1528,14 +1549,13 @@ namespace SourceGit.ViewModels
             if (_useAmend)
             {
                 var head = new Commands.QuerySingleCommit(_repo.FullPath, "HEAD").Result();
-                return new Commands.QueryStagedChangesWithAmend(_repo.FullPath, head.Parents.Count == 0 ? "4b825dc642cb6eb9a060e54bf8d69288fbee4904" : $"{head.SHA}^").Result();
+                return new Commands.QueryStagedChangesWithAmend(_repo.FullPath, head.Parents.Count == 0 ? Models.Commit.EmptyTreeSHA1 : $"{head.SHA}^").Result();
             }
 
             var rs = new List<Models.Change>();
             foreach (var c in _cached)
             {
-                if (c.Index != Models.ChangeState.None &&
-                    c.Index != Models.ChangeState.Untracked)
+                if (c.Index != Models.ChangeState.None)
                     rs.Add(c);
             }
             return rs;
@@ -1609,30 +1629,19 @@ namespace SourceGit.ViewModels
             {
                 await Task.Run(() => new Commands.Add(_repo.FullPath, _repo.IncludeUntracked).Use(log).Exec());
             }
-            else if (Native.OS.GitVersion >= Models.GitVersions.ADD_WITH_PATHSPECFILE)
-            {
-                var paths = new List<string>();
-                foreach (var c in changes)
-                    paths.Add(c.Path);
-
-                var tmpFile = Path.GetTempFileName();
-                File.WriteAllLines(tmpFile, paths);
-                await Task.Run(() => new Commands.Add(_repo.FullPath, tmpFile).Use(log).Exec());
-                File.Delete(tmpFile);
-            }
             else
             {
                 var paths = new List<string>();
                 foreach (var c in changes)
                     paths.Add(c.Path);
 
-                for (int i = 0; i < count; i += 10)
-                {
-                    var step = paths.GetRange(i, Math.Min(10, count - i));
-                    await Task.Run(() => new Commands.Add(_repo.FullPath, step).Use(log).Exec());
-                }
+                var pathSpecFile = Path.GetTempFileName();
+                await File.WriteAllLinesAsync(pathSpecFile, paths);
+                await Task.Run(() => new Commands.Add(_repo.FullPath, pathSpecFile).Use(log).Exec());
+                File.Delete(pathSpecFile);
             }
             log.Complete();
+
             _repo.MarkWorkingCopyDirtyManually();
             _repo.SetWatcherEnabled(true);
             IsStaging = false;
@@ -1656,19 +1665,23 @@ namespace SourceGit.ViewModels
                 log.AppendLine("$ git update-index --index-info ");
                 await Task.Run(() => new Commands.UnstageChangesForAmend(_repo.FullPath, changes).Exec());
             }
-            else if (count == _staged.Count)
-            {
-                await Task.Run(() => new Commands.Reset(_repo.FullPath).Use(log).Exec());
-            }
             else
             {
-                for (int i = 0; i < count; i += 10)
+                var paths = new List<string>();
+                foreach (var c in changes)
                 {
-                    var step = changes.GetRange(i, Math.Min(10, count - i));
-                    await Task.Run(() => new Commands.Reset(_repo.FullPath, step).Use(log).Exec());
+                    paths.Add(c.Path);
+                    if (c.Index == Models.ChangeState.Renamed)
+                        paths.Add(c.OriginalPath);
                 }
+
+                var pathSpecFile = Path.GetTempFileName();
+                await File.WriteAllLinesAsync(pathSpecFile, paths);
+                await Task.Run(() => new Commands.Restore(_repo.FullPath, pathSpecFile, true).Use(log).Exec());
+                File.Delete(pathSpecFile);
             }
             log.Complete();
+
             _repo.MarkWorkingCopyDirtyManually();
             _repo.SetWatcherEnabled(true);
             IsUnstaging = false;
@@ -1681,7 +1694,7 @@ namespace SourceGit.ViewModels
 
             if (change == null)
                 DetailContext = null;
-            else if (change.IsConflict && isUnstaged)
+            else if (change.IsConflicted)
                 DetailContext = new Conflict(_repo, this, change);
             else
                 DetailContext = new DiffContext(_repo.FullPath, new Models.DiffOption(change, isUnstaged), _detailContext as DiffContext);
@@ -1694,7 +1707,7 @@ namespace SourceGit.ViewModels
 
             if (!_repo.CanCreatePopup())
             {
-                App.RaiseException(_repo.FullPath, "Repository has unfinished job! Please wait!");
+                App.RaiseException(_repo.FullPath, "Repository has an unfinished job! Please wait!");
                 return;
             }
 
@@ -1718,6 +1731,7 @@ namespace SourceGit.ViewModels
             _repo.Settings.PushCommitMessage(_commitMessage);
             _repo.SetWatcherEnabled(false);
 
+            var signOff = _repo.Settings.EnableSignOffForCommit;
             var log = _repo.CreateLog("Commit");
             Task.Run(() =>
             {
@@ -1726,7 +1740,7 @@ namespace SourceGit.ViewModels
                     succ = new Commands.Add(_repo.FullPath, _repo.IncludeUntracked).Use(log).Exec();
 
                 if (succ)
-                    succ = new Commands.Commit(_repo.FullPath, _commitMessage, _useAmend, _repo.Settings.EnableSignOffForCommit).Use(log).Run();
+                    succ = new Commands.Commit(_repo.FullPath, _commitMessage, signOff, _useAmend, _resetAuthor).Use(log).Run();
 
                 log.Complete();
 
@@ -1764,13 +1778,16 @@ namespace SourceGit.ViewModels
             if (old.Count != cur.Count)
                 return true;
 
-            var oldSet = new HashSet<string>();
+            var oldMap = new Dictionary<string, Models.Change>();
             foreach (var c in old)
-                oldSet.Add($"{c.Path}\n{c.WorkTree}\n{c.Index}");
+                oldMap.Add(c.Path, c);
 
             foreach (var c in cur)
             {
-                if (!oldSet.Contains($"{c.Path}\n{c.WorkTree}\n{c.Index}"))
+                if (!oldMap.TryGetValue(c.Path, out var o))
+                    return true;
+
+                if (o.Index != c.Index || o.WorkTree != c.WorkTree)
                     return true;
             }
 
@@ -1783,6 +1800,7 @@ namespace SourceGit.ViewModels
         private bool _isUnstaging = false;
         private bool _isCommitting = false;
         private bool _useAmend = false;
+        private bool _resetAuthor = false;
         private bool _hasRemotes = false;
         private List<Models.Change> _cached = [];
         private List<Models.Change> _unstaged = [];
