@@ -1,3 +1,4 @@
+using Avalonia;
 using Avalonia.Controls;
 using Avalonia.Interactivity;
 using Avalonia.VisualTree;
@@ -40,5 +41,24 @@ namespace SourceGit.Views
             if (sender is TextDiffView textDiff)
                 BlockNavigationIndicator.Text = textDiff.BlockNavigation?.Indicator ?? string.Empty;
         }
+
+        private void OnUseFullTextDiffClicked(object sender, RoutedEventArgs e)
+        {
+            var textDiffView = this.FindDescendantOfType<TextDiffView>();
+            if (textDiffView == null)
+                return;
+
+            var presenter = textDiffView.FindDescendantOfType<ThemedTextDiffPresenter>();
+            if (presenter == null)
+                return;
+
+            if (presenter.DataContext is Models.TextDiff combined)
+                combined.ScrollOffset = Vector.Zero;
+            else if (presenter.DataContext is ViewModels.TwoSideTextDiff twoSides)
+                twoSides.File = string.Empty; // Just to reset `SyncScrollOffset` without affect UI refresh.
+
+            (DataContext as ViewModels.DiffContext)?.ToggleFullTextDiff();
+            e.Handled = true;
+        }
     }
 }
