@@ -126,7 +126,7 @@ namespace SourceGit.Views
                             typeface,
                             presenter.FontSize,
                             presenter.Foreground);
-                        context.DrawText(txt, new Point(Bounds.Width - txt.Width, y - txt.Height * 0.5));
+                        context.DrawText(txt, new Point(Bounds.Width - txt.Width, y - (txt.Height * 0.5)));
                     }
                 }
             }
@@ -212,7 +212,7 @@ namespace SourceGit.Views
                         }
 
                         if (indicator != null)
-                            context.DrawText(indicator, new Point(0, y - indicator.Height * 0.5));
+                            context.DrawText(indicator, new Point(0, y - (indicator.Height * 0.5)));
                     }
                 }
             }
@@ -1047,7 +1047,8 @@ namespace SourceGit.Views
                 // The first selected line (partial selection)
                 if (i == startIdx && startPosition.Column > 1)
                 {
-                    builder.AppendLine(line.Content.Substring(startPosition.Column - 1));
+                    builder.Append(line.Content.AsSpan(startPosition.Column - 1));
+                    builder.Append(Environment.NewLine);
                     continue;
                 }
 
@@ -1061,7 +1062,14 @@ namespace SourceGit.Views
                 // For the last line (selection range is within original source)
                 if (i == endIdx)
                 {
-                    builder.Append(endPosition.Column - 1 < line.Content.Length ? line.Content.Substring(0, endPosition.Column - 1) : line.Content);
+                    if (endPosition.Column - 1 < line.Content.Length)
+                    {
+                        builder.Append(line.Content.AsSpan(0, endPosition.Column - 1));
+                    }
+                    else
+                    {
+                        builder.Append(line.Content);
+                    }
                     break;
                 }
 
@@ -1251,7 +1259,7 @@ namespace SourceGit.Views
                 {
                     if (line.Content.Length > 10000)
                     {
-                        builder.Append(line.Content.Substring(0, 1000));
+                        builder.Append(line.Content.AsSpan(0, 1000));
                         builder.Append($"...({line.Content.Length - 1000} character trimmed)");
                     }
                     else
@@ -1494,7 +1502,7 @@ namespace SourceGit.Views
                 {
                     if (line.Content.Length > 10000)
                     {
-                        builder.Append(line.Content.Substring(0, 1000));
+                        builder.Append(line.Content.AsSpan(0, 1000));
                         builder.Append($"...({line.Content.Length - 1000} characters trimmed)");
                     }
                     else
@@ -1740,8 +1748,8 @@ namespace SourceGit.Views
                     return;
                 }
 
-                var top = chunk.Y + (chunk.Height >= 36 ? 16 : 4);
-                var right = (chunk.Combined || !chunk.IsOldSide) ? 16 : v.Bounds.Width * 0.5f + 16;
+                var top = chunk.Y + (chunk.Height >= 36 ? 8 : 2);
+                var right = (chunk.Combined || !chunk.IsOldSide) ? 26 : (v.Bounds.Width * 0.5f) + 26;
                 v.Popup.Margin = new Thickness(0, top, right, 0);
                 v.Popup.IsVisible = true;
             });
@@ -1860,7 +1868,7 @@ namespace SourceGit.Views
 
             if (!selection.HasLeftChanges)
             {
-                new Commands.Add(repo.FullPath, [change.Path]).Exec();
+                new Commands.Add(repo.FullPath, change).Exec();
             }
             else
             {
@@ -1921,7 +1929,7 @@ namespace SourceGit.Views
                 if (change.DataForAmend != null)
                     new Commands.UnstageChangesForAmend(repo.FullPath, [change]).Exec();
                 else
-                    new Commands.Reset(repo.FullPath, [change]).Exec();
+                    new Commands.Restore(repo.FullPath, change).Exec();
             }
             else
             {
